@@ -3,14 +3,12 @@ from __future__ import annotations
 import dataclasses
 import logging
 import pathlib
-import subprocess
 import typing
 
-from zntrack.core.parameter import ZnTrackOption
-from zntrack.utils import config
-from zntrack.utils.utils import module_handler
-
-from .jupyter import jupyter_class_to_file
+from zntrack import descriptor, utils
+from zntrack.core.jupyter import jupyter_class_to_file
+from zntrack.core.zntrackoption import ZnTrackOption
+from zntrack.zn.dependencies import NodeAttribute
 
 log = logging.getLogger(__name__)
 
@@ -34,7 +32,7 @@ def handle_deps(value) -> list:
         for lst_val in value:
             script += handle_deps(lst_val)
     else:
-        if isinstance(value, GraphWriter):
+        if isinstance(value, (GraphWriter, NodeAttribute)):
             for file in value.affected_files:
                 script += ["--deps", pathlib.Path(file).as_posix()]
         elif isinstance(value, (str, pathlib.Path)):
@@ -42,7 +40,7 @@ def handle_deps(value) -> list:
         elif value is None:
             pass
         else:
-            raise ValueError(f"Type {type(value)} is not supported!")
+            raise ValueError(f"Type {type(value)} ({value}) is not supported!")
 
     return script
 
@@ -80,12 +78,6 @@ class DVCRunOptions:
             value = getattr(self, field_name)
             if value:
                 out.append(f"--{field_name.replace('_', '-')}")
-            else:
-                if field_name == "no_exec":
-                    log.warning(
-                        "You will not be able to see the stdout/stderr "
-                        "of the process in real time!"
-                    )
         return out
 
 
@@ -107,44 +99,102 @@ def handle_dvc(value, dvc_args) -> list:
     return [f(x) for x in value for f in (option_func, posix_func)]
 
 
-@dataclasses.dataclass
-class DescriptorList:
-    """Dataclass to collect all descriptors of some parent class"""
+def filter_ZnTrackOption(
+    data,
+    cls,
+    zn_type: typing.Union[utils.ZnTypes, typing.List[utils.ZnTypes]],
+    return_with_type=False,
+    allow_none: bool = False,
+) -> dict:
+    """Filter the descriptor instances by zn_type
 
-    parent: GraphWriter
-    data: typing.List[ZnTrackOption] = dataclasses.field(default_factory=list)
+    Parameters
+    ----------
+    data: List[ZnTrackOption]
+        The ZnTrack options to query through
+    cls:
+        The instance the ZnTrack options are attached to
+    zn_type: str
+        The zn_type of the descriptors to gather
+    return_with_type: bool, default=False
+        return a dictionary with the Descriptor.dvc_option as keys
+    allow_none: bool, default=False
+        Use getattr(obj, name, None) instead of getattr(obj, name) to yield
+        None when an AttributeError occurs.
 
-    def filter(
-        self, zntrack_type: typing.Union[str, list], return_with_type=False
-    ) -> dict:
-        """Filter the descriptor instances by zntrack_type
+    Returns
+    -------
+    dict:
+        either {attr_name: attr_value}
+        or
+        {descriptor.dvc_option: {attr_name: attr_value}}
 
-        Parameters
-        ----------
-        zntrack_type: str
-            The zntrack_type of the descriptors to gather
-        return_with_type: bool, default=False
-            return a dictionary with the Descriptor.metadata.dvc_option as keys
+    """
+    if not isinstance(zn_type, list):
+        zn_type = [zn_type]
+    data = [x for x in data if x.zn_type in zn_type]
+    if return_with_type:
+        types_dict = {x.dvc_option: {} for x in data}
+        for entity in data:
+            if allow_none:
+                # avoid AttributeError
+                value = getattr(cls, entity.name, None)
+            else:
+                value = getattr(cls, entity.name)
+            types_dict[entity.dvc_option].update({entity.name: value})
+        return types_dict
+    if allow_none:
+        return {x.name: getattr(cls, x.name, None) for x in data}
+    # avoid AttributeError
+    return {x.name: getattr(cls, x.name) for x in data}
 
-        Returns
-        -------
-        dict:
-            either {attr_name: attr_value}
-            or
-            {descriptor.dvc_option: {attr_name: attr_value}}
 
-        """
-        if not isinstance(zntrack_type, list):
-            zntrack_type = [zntrack_type]
-        data = [x for x in self.data if x.metadata.zntrack_type in zntrack_type]
-        if return_with_type:
-            types_dict = {x.metadata.dvc_option: {} for x in data}
-            for entity in data:
-                types_dict[entity.metadata.dvc_option].update(
-                    {entity.name: getattr(self.parent, entity.name)}
-                )
-            return types_dict
-        return {x.name: getattr(self.parent, x.name) for x in data}
+def prepare_dvc_script(
+    node_name,
+    dvc_run_option: DVCRunOptions,
+    custom_args: list,
+    nb_name,
+    module,
+    func_or_cls,
+    call_args,
+) -> list:
+    """Prepate the dvc cmd to be called by subprocess
+
+    Parameters
+    ----------
+    node_name: str
+        Name of the Node
+    dvc_run_option: DVCRunOptions
+        dataclass to collect special DVC run options
+    custom_args: list[str]
+        all the params / deps / ... to be added to the script
+    nb_name: str|None
+        Notebook name for jupyter support
+    module: str like "src.my_module"
+    func_or_cls: str
+        The name of the Node class or function to be imported and run
+    call_args: str
+        Additional str like "(run_func=True)" or ".load().run_and_save"
+
+    Returns
+    -------
+
+    list[str]
+        The list to be passed to the subprocess call
+
+    """
+    script = ["dvc", "run", "-n", node_name]
+    script += dvc_run_option.dvc_args
+    script += custom_args
+
+    if nb_name is not None:
+        script += ["--deps", utils.module_to_path(module).as_posix()]
+
+    import_str = f"""{utils.get_python_interpreter()} -c "from {module} import """
+    import_str += f"""{func_or_cls}; {func_or_cls}{call_args}" """
+    script += [import_str]
+    log.debug(f"dvc script: {' '.join([str(x) for x in script])}")
+    return script
 
 
 class GraphWriter:
@@ -158,18 +208,11 @@ class GraphWriter:
 
     def __init__(self, **kwargs):
         self.node_name = kwargs.get("name", None)
-        for data in self._descriptor_list.data:
-            if data.metadata.zntrack_type == "deps":
-                data.update_default()
 
     @property
-    def _descriptor_list(self) -> DescriptorList:
+    def _descriptor_list(self) -> typing.List[ZnTrackOption]:
         """Get all descriptors of this instance"""
-        descriptor_list = []
-        for option in vars(type(self)).values():
-            if isinstance(option, ZnTrackOption):
-                descriptor_list.append(option)
-        return DescriptorList(parent=self, data=descriptor_list)
+        return descriptor.get_descriptors(self, ZnTrackOption)
 
     @property
     def node_name(self) -> str:
@@ -194,7 +237,7 @@ class GraphWriter:
         this can be changed when using nb_mode
         """
         if self._module is None:
-            return module_handler(self.__class__)
+            return utils.module_handler(self.__class__)
         return self._module
 
     def save(self, results: bool = False):
@@ -214,54 +257,37 @@ class GraphWriter:
     def affected_files(self) -> typing.Set[pathlib.Path]:
         """list of all files that can be changed by this instance"""
         files = []
-        for option in self._descriptor_list.data:
+        for option in self._descriptor_list:
             file = option.get_filename(self)
-            if file.tracked:
-                files.append(file.path)
-            elif file.value_tracked:
+            if option.zn_type in utils.VALUE_DVC_TRACKED:
+                files.append(file)
+            elif option.zn_type in utils.FILE_DVC_TRACKED:
                 value = getattr(self, option.name)
-                if isinstance(value, list):
+                if isinstance(value, (list, tuple)):
                     files += value
                 else:
                     files.append(value)
+            # deps or params are not affected files
 
         files = [x for x in files if x is not None]
         return set(files)
 
-    @property
-    def python_interpreter(self) -> str:
-        """Find the most suitable python interpreter
-
-        Try to run subprocess check calls to see, which python interpreter
-        should be selected
-
-        Returns
-        -------
-        interpreter: str
-            Name of the python interpreter that works with subprocess calls
+    @classmethod
+    def convert_notebook(cls, nb_name: str = None):
+        """Use jupyter_class_to_file to convert ipynb to py
 
+        Parameters
+        ----------
+        nb_name: str
+            Notebook name when not using config.nb_name (this is not recommended)
         """
-
-        for interpreter in ["python3", "python"]:
-            try:
-                subprocess.check_call(
-                    [interpreter, "--version"],
-                    stdout=subprocess.DEVNULL,
-                    stderr=subprocess.DEVNULL,
-                )
-                log.debug(f"Using command {interpreter} for dvc!")
-                return interpreter
-
-            except subprocess.CalledProcessError:
-                log.debug(f"{interpreter} is not working!")
-        raise ValueError(
-            "Could not find a working python interpreter to work with subprocesses!"
-        )
+        jupyter_class_to_file(nb_name=nb_name, module_name=cls.__name__)
 
     def write_graph(
         self,
         silent: bool = False,
         nb_name: str = None,
+        notebook: bool = True,
         no_commit: bool = False,
         external: bool = False,
         always_changed: bool = False,
@@ -283,6 +309,8 @@ class GraphWriter:
             subprocess call.
         nb_name: str
             Notebook name when not using config.nb_name (this is not recommended)
+        notebook: bool, default = True
+            convert the notebook to a py File
         no_commit: dvc parameter
         external: dvc parameter
         always_changed: dvc parameter
@@ -299,75 +327,79 @@ class GraphWriter:
         Use 'dvc status' to check, if the stage needs to be rerun.
 
         """
+
+        if silent:
+            log.warning(
+                "DeprecationWarning: silent was replaced by 'zntrack.config.log_level ="
+                " logging.ERROR'"
+            )
         if run is not None:
             no_exec = not run
 
-        if nb_name is None:
-            nb_name = config.nb_name
+        log.debug("--- Writing new DVC file ---")
 
-        # Jupyter Notebook
-        if nb_name is not None:
-            self._module = f"{config.nb_class_path}.{self.__class__.__name__}"
-
-            jupyter_class_to_file(
-                silent=silent, nb_name=nb_name, module_name=self.__class__.__name__
-            )
-
-        self.save()
-
-        if not silent:
-            log.warning("--- Writing new DVC file! ---")
-
-        script = ["dvc", "run", "-n", self.node_name]
-
-        script += DVCRunOptions(
+        dvc_run_option = DVCRunOptions(
             no_commit=no_commit,
             external=external,
             always_changed=always_changed,
             no_run_cache=no_run_cache,
             no_exec=no_exec,
             force=force,
-        ).dvc_args
+        )
 
         # Jupyter Notebook
+        nb_name = utils.update_nb_name(nb_name)
         if nb_name is not None:
-            script += [
-                "--deps",
-                pathlib.Path(*self.module.split(".")).with_suffix(".py").as_posix(),
-            ]
+            self._module = f"{utils.config.nb_class_path}.{self.__class__.__name__}"
+            if notebook:
+                self.convert_notebook(nb_name)
 
+        custom_args = []
         # Handle Parameter
-        if len(self._descriptor_list.filter(zntrack_type=["params", "method"])) > 0:
-            script += [
+        params_list = filter_ZnTrackOption(
+            data=self._descriptor_list, cls=self, zn_type=[utils.ZnTypes.PARAMS]
+        )
+        if len(params_list) > 0:
+            custom_args += [
                 "--params",
-                f"params.yaml:{self.node_name}",
+                f"{utils.Files.params}:{self.node_name}",
             ]
         zn_options_set = set()
-        for option in self._descriptor_list.data:
-            value = getattr(self, option.name)
-            if option.metadata.zntrack_type == "dvc":
-                script += handle_dvc(value, option.metadata.dvc_args)
+        for option in self._descriptor_list:
+            if option.zn_type == utils.ZnTypes.DVC:
+                value = getattr(self, option.name)
+                custom_args += handle_dvc(value, option.dvc_args)
             # Handle Zn Options
-            elif option.metadata.zntrack_type in ["zn", "metadata"]:
+            elif option.zn_type in [
+                utils.ZnTypes.RESULTS,
+                utils.ZnTypes.METADATA,
+            ]:
                 zn_options_set.add(
                     (
-                        f"--{option.metadata.dvc_args}",
-                        option.get_filename(self).path.as_posix(),
+                        f"--{option.dvc_args}",
+                        option.get_filename(self).as_posix(),
                     )
                 )
-            elif option.metadata.zntrack_type == "deps":
-                script += handle_deps(value)
+            elif option.zn_type == utils.ZnTypes.DEPS:
+                value = getattr(self, option.name)
+                custom_args += handle_deps(value)
 
         for pair in zn_options_set:
-            script += pair
+            custom_args += pair
+
+        script = prepare_dvc_script(
+            node_name=self.node_name,
+            dvc_run_option=dvc_run_option,
+            custom_args=custom_args,
+            nb_name=nb_name,
+            module=self.module,
+            func_or_cls=self.__class__.__name__,
+            call_args=f".load(name='{self.node_name}').run_and_save()",
+        )
 
         # Add command to run the script
-        cls_name = self.__class__.__name__
-        script.append(
-            f"""{self.python_interpreter} -c "from {self.module} import {cls_name}; """
-            f"""{cls_name}.load(name='{self.node_name}').run_and_save()" """
-        )
-        log.debug(f"running script: {' '.join([str(x) for x in script])}")
+
+        self.save()
 
         log.debug(
             "If you are using a jupyter notebook, you may not be able to see the "
@@ -376,10 +408,4 @@ class GraphWriter:
 
         if dry_run:
             return script
-        else:
-            process = subprocess.run(script, capture_output=True, check=True)
-            if not silent:
-                if len(process.stdout) > 0:
-                    log.info(process.stdout.decode())
-                if len(process.stderr) > 0:
-                    log.warning(process.stderr.decode())
+        utils.run_dvc_cmd(script)
