@@ -1,32 +1,40 @@
 import logging
-
-log = logging.getLogger(__name__)
 import pathlib
 import re
 import subprocess
+from functools import lru_cache
 
 from zntrack.utils.config import config
 
+log = logging.getLogger(__name__)
 
-def jupyter_class_to_file(silent, nb_name, module_name):
+
+@lru_cache(None)
+def log_jupyter_warning():
+    """Use lru_cache to only print this once"""
+    log.warning(
+        "Jupyter support is an experimental feature! Please save your "
+        "notebook before running this command!\n"
+        "Submit issues to https://github.com/zincware/ZnTrack."
+    )
+
+
+def jupyter_class_to_file(nb_name, module_name):
     """Extract the class definition form an ipynb file"""
+    # TODO is it really module_name and not class name?
 
-    if not silent:
-        log.warning(
-            "Jupyter support is an experimental feature! Please save your "
-            "notebook before running this command!\n"
-            "Submit issues to https://github.com/zincware/ZnTrack."
-        )
-        log.warning(f"Converting {nb_name} to file {module_name}.py")
-
+    log_jupyter_warning()
+    log.debug(f"Converting {nb_name} to file {module_name}.py")
     nb_name = pathlib.Path(nb_name)
 
     subprocess.run(
         ["jupyter", "nbconvert", "--to", "script", nb_name],
-        capture_output=not silent,
+        capture_output=config.log_level > logging.INFO,
+        check=True,
     )
 
     reading_class = False
+    found_node = False
 
     imports = ""
 
@@ -52,6 +60,15 @@ def jupyter_class_to_file(silent, nb_name, module_name):
             if line.startswith("@"):  # handle decorators
                 reading_class = True
                 class_definition += line
+            if line.startswith(f"class {module_name}") or line.startswith(
+                f"def {module_name}"
+            ):
+                found_node = True
+            if found_node and not reading_class:
+                if re.match(r"#.*zntrack:.*break", line):
+                    # stop converting the file after this line if the Node was already
+                    #  found
+                    break
 
     src = imports + "\n\n" + class_definition
 
