@@ -1,17 +1,9 @@
-"""
-This program and the accompanying materials are made available under the terms of the
-Eclipse Public License v2.0 which accompanies this distribution, and is available at
-https://www.eclipse.org/legal/epl-v20.html
-SPDX-License-Identifier: EPL-2.0
-
-Copyright Contributors to the Zincware Project.
-
-Description: The class for the ZnTrackProject
-"""
+"""The class for the ZnTrackProject"""
 import logging
 import subprocess
 from datetime import datetime
 
+from zntrack import utils
 from zntrack.interface import DVCInterface
 
 log = logging.getLogger(__name__)
@@ -40,7 +32,7 @@ class ZnTrackProject(DVCInterface):
         log.info("Running git add")
         subprocess.check_call(["git", "add", "."])
         log.info("Queue DVC stage")
-        subprocess.check_call(["dvc", "exp", "run", "--name", self.name, "--queue"])
+        utils.run_dvc_cmd(["dvc", "exp", "run", "--name", self.name, "--queue"])
 
     @staticmethod
     def remove_queue():
@@ -52,7 +44,7 @@ class ZnTrackProject(DVCInterface):
     def run_all():
         """Run all queried experiments"""
         log.info("RUN DVC stage")
-        subprocess.check_call(["dvc", "exp", "run", "--run-all"])
+        utils.run_dvc_cmd(["dvc", "exp", "run", "--run-all"])
         log.info("Running git add")
         subprocess.check_call(["git", "add", "."])
 
@@ -71,6 +63,7 @@ class ZnTrackProject(DVCInterface):
         for trial in range(3):
             try:
                 subprocess.check_call(["dvc", "exp", "apply", self.name])
+                break
             except subprocess.CalledProcessError as error:
                 # sometimes it takes more than one trial (windows)
                 if trial == 2:
@@ -110,4 +103,4 @@ class ZnTrackProject(DVCInterface):
     @staticmethod
     def repro():
         """Run dvc repro"""
-        subprocess.check_call(["dvc", "repro"])
+        utils.run_dvc_cmd(["dvc", "repro"])
