@@ -3,23 +3,19 @@ import pathlib
 
 import pandas as pd
 
-from zntrack.core.parameter import File, ZnTrackOption
-from zntrack.descriptor import Metadata
+from zntrack import utils
+from zntrack.core.zntrackoption import ZnTrackOption
 
 log = logging.getLogger(__name__)
 
 
 class plots(ZnTrackOption):
-    metadata = Metadata(dvc_option="plots_no_cache", zntrack_type="zn")
+    dvc_option = utils.DVCOptions.PLOTS_NO_CACHE.value
+    zn_type = utils.ZnTypes.RESULTS
 
-    def get_filename(self, instance) -> File:
+    def get_filename(self, instance) -> pathlib.Path:
         """Overwrite filename to csv"""
-        return File(
-            path=pathlib.Path(
-                "nodes", instance.node_name, f"{self.metadata.dvc_option}.csv"
-            ),
-            tracked=True,
-        )
+        return pathlib.Path("nodes", instance.node_name, f"{self.dvc_option}.csv")
 
     def save(self, instance):
         """Save value with pd.DataFrame.to_csv"""
@@ -40,13 +36,11 @@ class plots(ZnTrackOption):
             )
 
         file = self.get_filename(instance)
-        file.path.parent.mkdir(exist_ok=True, parents=True)
-        value.to_csv(file.path)
+        file.parent.mkdir(exist_ok=True, parents=True)
+        value.to_csv(file)
 
-    def load(self, instance):
+    def get_data_from_files(self, instance):
         """Load value with pd.read_csv"""
+
         file = self.get_filename(instance)
-        try:
-            instance.__dict__.update({self.name: pd.read_csv(file.path, index_col=0)})
-        except (FileNotFoundError, KeyError):
-            pass
+        return pd.read_csv(file, index_col=0)
